@@ -57,6 +57,7 @@ struct _BjbMainToolbar
   /* Main View */
   GtkWidget *new_button;
   GtkWidget *back_button;
+  GtkWidget *title_entry;
   GtkWidget *list_button;
   GtkWidget *grid_button;
   GtkWidget *select_button;
@@ -385,11 +386,13 @@ on_note_renamed (BijiItem *note,
   const gchar *str;
 
   str = biji_item_get_title (note);
-  if (str == NULL)
+  if (str == NULL || strlen(str) == 0)
     str = _("Untitled");
 
   gtk_header_bar_set_title (GTK_HEADER_BAR (self), str);
   gtk_header_bar_set_subtitle (GTK_HEADER_BAR (self), NULL);
+
+  gtk_entry_set_text (GTK_ENTRY (self->title_entry), str);
 }
 
 static void
@@ -432,6 +435,15 @@ on_last_updated_cb (BijiItem       *note,
   gtk_label_set_text (GTK_LABEL (self->last_update_item), label);
 }
 
+static void
+on_title_changed (BjbMainToolbar *self,
+                  GtkEntry       *title)
+{
+  const char *str = gtk_entry_get_text (title);
+  if (strlen (str) > 0)
+    biji_note_obj_set_title (self->note, str);
+}
+
 static void
 populate_bar_for_note_view (BjbMainToolbar *self)
 {
@@ -447,6 +459,8 @@ populate_bar_for_note_view (BjbMainToolbar *self)
 
   settings = bjb_app_get_settings (g_application_get_default());
 
+  gtk_header_bar_set_custom_title (GTK_HEADER_BAR (self), self->title_entry);
+
   gtk_widget_hide (self->new_button);
   gtk_widget_hide (self->style_buttons);
   gtk_widget_hide (self->search_button);
@@ -495,6 +509,7 @@ populate_bar_for_note_view (BjbMainToolbar *self)
 static void
 bjb_main_toolbar_reset (BjbMainToolbar *self)
 {
+  gtk_header_bar_set_custom_title (GTK_HEADER_BAR (self), NULL);
   gtk_header_bar_set_show_close_button (GTK_HEADER_BAR (self), TRUE);
 
   gtk_widget_show (self->button_stack);
@@ -643,18 +658,6 @@ populate_main_toolbar(BjbMainToolbar *self)
   }
 }
 
-static void
-bjb_main_toolbar_setup_menu (BjbMainToolbar *self)
-{
-  gtk_widget_add_accelerator (self->undo_item, "activate", self->accel, GDK_KEY_z,
-                              GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
-  gtk_widget_add_accelerator (self->redo_item, "activate", self->accel, GDK_KEY_z,
-                              GDK_CONTROL_MASK | GDK_SHIFT_MASK, GTK_ACCEL_VISIBLE);
-  gtk_widget_add_accelerator (self->trash_item, "activate", self->accel,
-                              GDK_KEY_Delete, GDK_CONTROL_MASK,
-                              GTK_ACCEL_VISIBLE);
-}
-
 static void
 bjb_main_toolbar_constructed (GObject *obj)
 {
@@ -664,8 +667,9 @@ bjb_main_toolbar_constructed (GObject *obj)
   gtk_window_add_accel_group (self->window, self->accel);
   gtk_widget_add_accelerator (self->new_button, "clicked", self->accel, GDK_KEY_n,
                               GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
+  gtk_widget_add_accelerator (self->search_button, "clicked", self->accel, GDK_KEY_f,
+                              GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
 
-  bjb_main_toolbar_setup_menu (self);
   g_signal_connect_swapped (self->window, "view-changed",
                             G_CALLBACK (populate_main_toolbar), self);
 
@@ -780,6 +784,7 @@ bjb_main_toolbar_class_init (BjbMainToolbarClass *klass)
   gtk_widget_class_bind_template_child (widget_class, BjbMainToolbar, button_stack);
   gtk_widget_class_bind_template_child (widget_class, BjbMainToolbar, new_button);
   gtk_widget_class_bind_template_child (widget_class, BjbMainToolbar, back_button);
+  gtk_widget_class_bind_template_child (widget_class, BjbMainToolbar, title_entry);
   gtk_widget_class_bind_template_child (widget_class, BjbMainToolbar, list_button);
   gtk_widget_class_bind_template_child (widget_class, BjbMainToolbar, grid_button);
   gtk_widget_class_bind_template_child (widget_class, BjbMainToolbar, search_button);
@@ -804,6 +809,8 @@ bjb_main_toolbar_class_init (BjbMainToolbarClass *klass)
   gtk_widget_class_bind_template_callback (widget_class, on_view_mode_clicked);
   gtk_widget_class_bind_template_callback (widget_class, on_empty_clicked_callback);
   gtk_widget_class_bind_template_callback (widget_class, on_color_button_clicked);
+
+  gtk_widget_class_bind_template_callback (widget_class, on_title_changed);
 }
 
 BjbMainToolbar *
@@ -824,3 +831,9 @@ bjb_main_toolbar_new (BjbMainView *parent,
   populate_main_toolbar(self);
   return self;
 }
+
+void
+bjb_main_toolbar_title_focus (BjbMainToolbar *self)
+{
+  gtk_entry_grab_focus_without_selecting (GTK_ENTRY (self->title_entry));
+}
